#include "ExampleFrameListener.h"

#include <iostream>
#include <cassert>

using namespace std;

#include <Ogre.h>

using namespace Ogre;

#include <btBulletCollisionCommon.h>

#include "OgreToBtMeshConverter.h"
#include "Player.h"

#define PRINT_BTVECTOR3(btvec)						\
  cout << btvec[0] << " " << btvec[1] << " " << btvec[2] << endl;

//
// Extends the ExampleFrameListener for input and 
// collision detection duties.
//
class MyFrameListener : public ExampleFrameListener
{
 protected:
  SceneManager* mSceneMgr;
   
  // all the required objects needed to set up Bullet
  // for collision detection 
  btDefaultCollisionConfiguration* mColConfig; 
  btCollisionWorld* mColWorld;
  btCollisionDispatcher* mDispatcher;
  btAxisSweep3* mBroadphase; 
   
  // our players character 
  Player* player;
    
 public:
 MyFrameListener(
		 RenderWindow* win, Camera* cam, SceneManager* sceneMgr,
		 bool bufferedKeys = false, bool bufferedMouse = false
		 ) 
   : ExampleFrameListener(win, cam, bufferedKeys, bufferedMouse), mSceneMgr(sceneMgr) 
  {
    //
    // initialize bullets collision detection system
    //
    btVector3 worldAabbMin(-1000,-1000,-1000);
    btVector3 worldAabbMax(1000,1000,1000);
            
    mBroadphase = new btAxisSweep3(worldAabbMin, worldAabbMax); // broadphase
    mColConfig = new btDefaultCollisionConfiguration();
    mDispatcher = new btCollisionDispatcher(mColConfig); // narrowphase pair-wise checking
    mColWorld = new btCollisionWorld(mDispatcher, mBroadphase, mColConfig);
           
    //
    // Registers every mesh in OGREs scene manager with the
    // collision world. You should be adding your environment
    // to the scene manager before this call. Your moveable
    // character should be created after this call as to avoid
    // having his mesh become part of the static level geometry.
    //
    // All meshes here are placed into group 2 and set to
    // collide with group 1. You character should be in group
    // 1, and set to collide with group 2. More about contact groups
    // in the manual.
    registerAllEntitiesAsColliders(sceneMgr, mColWorld);
            
            
    // Create our player object (you will need to fill in much
    // of this class). 
    player = new Player(mSceneMgr, mColWorld);
    player->init();
           
    //
    // Debug code. Enable to make sure that your level is
    // properly loaded with the collision world.
    //

    /*
      ManualObject* myManualObject = new ManualObject("manual1"); 
      SceneNode* myManualObjectNode = mSceneMgr->getRootSceneNode()->createChildSceneNode("manual1_node"); 

      MaterialPtr myManualObjectMaterial = MaterialManager::getSingleton().create("manual1Material","debugger"); 
      myManualObjectMaterial->setReceiveShadows(false); 
      myManualObjectMaterial->getTechnique(0)->setLightingEnabled(true); 
      myManualObjectMaterial->getTechnique(0)->getPass(0)->setDiffuse(0,0,1,0); 
      myManualObjectMaterial->getTechnique(0)->getPass(0)->setAmbient(0,0,1); 
      myManualObjectMaterial->getTechnique(0)->getPass(0)->setSelfIllumination(0,0,1); 
            
      myManualObject->begin("manual1Material", Ogre::RenderOperation::OT_LINE_LIST); 
      myManualObject->position(x0[0], x0[1], x0[2]);
      myManualObject->position(x1[0], x1[1], x1[2]);
      myManualObject->end();
            
      myManualObjectNode->attachObject(myManualObject);
            
      const btCollisionObjectArray&  objarr = mColWorld->getCollisionObjectArray();
      for (int z = 0; z < objarr.size(); ++z) {
      ManualObject* myManualObject = new ManualObject(String("manual") + StringConverter::toString(z)); 
      SceneNode* myManualObjectNode = mSceneMgr->getRootSceneNode()->createChildSceneNode(String("manual") + StringConverter::toString(z)); 
    
      MaterialPtr myManualObjectMaterial = MaterialManager::getSingleton().create(String("manualmat") + StringConverter::toString(z),"debugger"); 
      myManualObjectMaterial->setReceiveShadows(false); 
      myManualObjectMaterial->getTechnique(0)->setLightingEnabled(true); 
      myManualObjectMaterial->getTechnique(0)->getPass(0)->setDiffuse(0,0,1,0); 
      myManualObjectMaterial->getTechnique(0)->getPass(0)->setAmbient(0,0,1); 
      myManualObjectMaterial->getTechnique(0)->getPass(0)->setSelfIllumination(0,0,1); 
                
      btCollisionObject* obj = objarr[z];
      btCollisionShape* shape = obj->getCollisionShape();
      if (shape->getShapeType() == 16) {
      btBvhTriangleMeshShape* trishape = static_cast<btBvhTriangleMeshShape*>(shape);
      btTriangleMesh* trimesh = static_cast<btTriangleMesh*>(trishape->getMeshInterface());
                    
      unsigned char *verts, *ind;
      int nverts, nfaces;
      int vstride, istride;
      PHY_ScalarType vtype, itype;
      cout << vtype << endl;
                    
      trimesh->getLockedVertexIndexBase(&verts, nverts, vtype, vstride, &ind, istride, nfaces, itype);
      btVector3 scale = trimesh->getScaling();
                    
      myManualObject->begin("manual1Material", Ogre::RenderOperation::OT_LINE_LIST);
      for (int m = 0; m < nfaces; ++m) {
      //cout << "vtype: " << vtype << endl;
      //cout << "itype: " << itype << endl;
                        
      int* wootind = (int*)(ind + m * istride);
      //cout << wootind[0] << " " << wootind[1] << " " << wootind[2] << endl;
                        
                        
      float* vert = (float*)(verts +wootind[0] * vstride);
      myManualObject->position(scale[0] * vert[0], scale[1] * vert[1], scale[2] * vert[2]);
                        
      vert = (float*)(verts +wootind[1] * vstride);
      myManualObject->position(scale[0] * vert[0], scale[1] * vert[1], scale[2] * vert[2]);
                        
      vert = (float*)(verts +wootind[1] * vstride);
      myManualObject->position(scale[0] * vert[0], scale[1] * vert[1], scale[2] * vert[2]);
                        
      vert = (float*)(verts +wootind[2] * vstride);
      myManualObject->position(scale[0] * vert[0], scale[1] * vert[1], scale[2] * vert[2]);
                        
      vert = (float*)(verts +wootind[2] * vstride);
      myManualObject->position(scale[0] * vert[0], scale[1] * vert[1], scale[2] * vert[2]);
                        
      vert = (float*)(verts +wootind[0] * vstride);
      myManualObject->position(scale[0] * vert[0], scale[1] * vert[1], scale[2] * vert[2]);
                        
      }
      myManualObject->end();
      }
      myManualObjectNode->attachObject(myManualObject);
      btTransform trans = obj->getWorldTransform();
      btVector3 origin = trans.getOrigin();
      btQuaternion quat = trans.getRotation();

      cout << "origin: " << origin[0] << " " << origin[1] << " " << origin[2] << endl;
      cout << quat[0] << " " << quat[1] << " " << quat[2] << " " << quat[3] << endl; 
      myManualObjectNode->setPosition(origin[0], origin[1], origin[2]);
                
      myManualObjectNode->setOrientation(
      Quaternion(quat[3], quat[0], quat[1], quat[2]));
                
      //myManualObjectNode->setOrientation(Quaternion(Degree(-90), Vector3(1.0, 0.0, 0.0)));
      myManualObjectNode->setInheritOrientation(true);
      }
    */
  }

  virtual ~MyFrameListener() 
    {
      // dispose of our allocated collision stuff
      delete mColWorld;
      delete mDispatcher;
      delete mColConfig;
      delete mBroadphase;
    }

  bool frameStarted(const FrameEvent& evt)
  {
    // do whatever the example frame listener did for frame started
    // processUnbufferedKeyInput is called in here
    if (!ExampleFrameListener::frameStarted(evt)) return false;
        
    // Here's where all the collision detection magic occurs. 
    // We need to call this before performing our character cylinder 
    // world check. We do not need to call this function if we wish to simply
    // perform ray queries. This is here to satisfy requirement 3
    // of the lab.
    mColWorld->performDiscreteCollisionDetection();

    // notify the player of the start of a frame
    player->updateFrameBegin(evt);
        
    return true;
  }
    
  bool frameEnded(const FrameEvent& evt)
  {
    // call the ExampleFrameListener version of frame ended
    if (!ExampleFrameListener::frameEnded(evt)) return false;
        
    // notify the player of the end of the frame
    player->updateFrameEnd(evt);

    return true;
  }
    
  bool processUnbufferedKeyInput(const FrameEvent& evt)
  {
    // process all of the input that ExampleFrameListener would
    // normally process
    if (!ExampleFrameListener::processUnbufferedKeyInput(evt)) return false;
       
    //
    // The keys have been querried for you already. You will need
    // to implement move, however.
    // 
    if (mKeyboard->isKeyDown(OIS::KC_I)) {
      player->move(Player::FORWARD);
    }
        
    if (mKeyboard->isKeyDown(OIS::KC_K)) {
      player->move(Player::BACKWARD);
    } 
        
    if (mKeyboard->isKeyDown(OIS::KC_J)) {
      player->move(Player::TURN_LEFT);
    }
    if (mKeyboard->isKeyDown(OIS::KC_L)) {
      player->move(Player::TURN_RIGHT);
    }
        
    return true;
  }
};
