#include "Player.h"

using namespace Ogre;

//
// Collects ray contact information. This object will pretty much
// only record the closest contact point.
//
struct  AllRayResultCallback : public btCollisionWorld::RayResultCallback
{
  AllRayResultCallback(const btVector3& rayFromWorld,
		       const btVector3& rayToWorld)
    : m_rayFromWorld(rayFromWorld), m_rayToWorld(rayToWorld), m_closestHitFraction(1.0)
  {
  }

  btVector3   m_rayFromWorld;
  btVector3   m_rayToWorld;
  btVector3   m_hitNormalWorld;
  btVector3   m_hitPointWorld;
  float m_closestHitFraction;
        
  virtual btScalar AddSingleResult(btCollisionWorld::LocalRayResult& rayResult,
				   bool normalInWorldSpace)
  {
    // Is the new result closer than the last. If so, save as the
    // closest
    if (rayResult.m_hitFraction < m_closestHitFraction)
      m_closestHitFraction = rayResult.m_hitFraction;

    m_collisionObject = rayResult.m_collisionObject;
    if (normalInWorldSpace) {
      m_hitNormalWorld = rayResult.m_hitNormalLocal;
    } 
    else {
      ///need to transform normal into worldspace
      m_hitNormalWorld = m_collisionObject->getWorldTransform().getBasis()*rayResult.m_hitNormalLocal;
    }
        
    m_hitPointWorld.setInterpolate3(m_rayFromWorld,m_rayToWorld,m_closestHitFraction);
    return 1.f;
  }
};


Player::Player(SceneManager* sceneMgr, btCollisionWorld* btColWorld) : 
  mSceneMgr(sceneMgr), mColWorld(btColWorld) 
{
}
	
Player::~Player() 
{
}
	
void Player::init() 
{
  //
  // create your entities and scene nodes here
  //
    
    


  //
  // For part 3 on the lab page, create your cylinder volume 
  // here.
  //
  // Basic steps:
  //
  // 1.) create the shape
  // 2.) create and set the collision object
  // 3.) add the collision object to the collision world
  //
  // Example:
  //
  //
  // Vector3 aabbExtents = charEnt->getBoundingBox().getHalfSize();
  // btVector3 cylHalfExtents(aabbExtents[0] * scale * .8, aabbExtents[1] * 0.25 * scale, aabbExtents[2] * scale * .8);
  // mCylShape = new btCylinderShape(cylHalfExtents);
  // mCylColObj = new btCollisionObject();
  // mCylColObj->setCollisionShape(mCylShape);
  //
  // btTransform trans;
  // trans.setIdentity();
  // trans.setOrigin(btVector3(0.0, aabbExtents[1] * scale, 0.0));
  // mCylColObj->setWorldTransform(trans);
  // mColWorld->addCollisionObject(mCylColObj, 1, 2);
}

// Move your character according to the direction specified
void Player::move(direction dir)
{
  // You will not only need to move your scene node, but the
  // collision volumes as well. Then you will need to do some
  // collision checks to determine if the move is valid. If not, you
  // should move back to your last state in which you were not
  // colliding.
    
  //
  // For part 2 you will need to fire a ray using btCollisionWorld.
  // This is relatively simple.
  //
  // btVector from, to;
  // 
  // ...
  //
  // AllRayResultCallback myCback(from, to);
  // mColWorld->rayTest(from, to, myCback, 2);
  //
  // You can get the point of closest collision through
  // 
  // myCback.m_hitPointWorld 
  // 
  // after the call to rayTest
  //
    
  // For part 3 you can determine if you have made any contacts with
  // the environment through the following sample code. Note: this
  // contact information is generated from the
  // mColWorld->performDiscreteCollisionDetection() call in
  // MyFrameListener::frameStarted code and you are simply querying
  // the results.
  //
  //
  // int nContact = 0;
  // int nManifolds = mColWorld->getDispatcher()->getNumManifolds();
  //
  // for (int i = 0; i < nManifolds; ++i) {
  //     btPersistentManifold* contactManifold = mColWorld->
  //         getDispatcher()->getManifoldByIndexInternal(i);
  //     nContact += contactManifold->getNumContacts();
  // }
  //
  // This will tell you the total number of geometry intersections
  // in the world. Given that the world meshes are set up as to only
  // contact with the player (see collision groups in the manual), 
  // all of the contact points generated
  // should be between our player and the world. To query the
  // contact point count specifically for our character, you can
  // look at the btCollisionObject in collision using 
  //
  // contactManifold->getBody0()
  //
  // and
  //
  // contactManifold->getBody1()
  //
  // Use the API reference guide to get more information on these
  // functions.
  //
  // If you set up everything correctly,
  // it should only be >0 when your characters cylinder volume is in
  // contact with the world.
}
   
void Player::updateFrameBegin(const FrameEvent& evt)
{
  evtFrameBegin = evt;
}

void Player::updateFrameEnd(const FrameEvent& evt)
{
  evtFrameEnd = evt;       
}

